@@ -255,9 +255,9 @@ on_saltstack = 'SALT_ON_SALTSTACK' in os.environ
 project = 'Salt'
 
 version = salt.version.__version__
-latest_release = '2019.2.0'  # latest release
-previous_release = '2018.3.4'  # latest release from previous branch
-previous_release_dir = '2018.3'  # path on web server for previous branch
+latest_release = os.environ.get('LATEST_RELEASE', 'latest_release')  # latest release (2019.2.3)
+previous_release = os.environ.get('PREVIOUS_RELEASE', 'previous_release')  # latest release from previous branch (2018.3.5)
+previous_release_dir = os.environ.get('PREVIOUS_RELEASE_DIR', 'previous_release_dir')  # path on web server for previous branch (2018.3)
 next_release = ''  # next release
 next_release_dir = ''  # path on web server for next release branch
 
@@ -268,22 +268,35 @@ if on_saltstack:
     copyright = time.strftime("%Y")
 
 # < --- START do not merge these settings to other branches START ---> #
-build_type = 'latest'  # latest, previous, develop, next
-release = latest_release
+build_type = os.environ.get('BUILD_TYPE', 'master') # latest, previous, master, next
 # < --- END do not merge these settings to other branches END ---> #
 
 # Set google custom search engine
 
-if release == latest_release:
+if build_type == 'master':
+    release = latest_release
+    search_cx = '011515552685726825874:v1had6i279q' # master
+    #search_cx = '011515552685726825874:x17j5zl74g8' # develop
+elif build_type == 'next':
+    release = next_release
+    search_cx = '011515552685726825874:ht0p8miksrm' # latest
+elif build_type == 'previous':
+    release = previous_release
+    if release.startswith('3000'):
+        search_cx = '011515552685726825874:3skhaozjtyn' # 3000
+    elif release.startswith('2019.2'):
+        search_cx = '011515552685726825874:huvjhlpptnm' # 2019.2
+    elif release.startswith('2018.3'):
+        search_cx = '011515552685726825874:vadptdpvyyu' # 2018.3
+    elif release.startswith('2017.7'):
+        search_cx = '011515552685726825874:w-hxmnbcpou' # 2017.7
+    elif release.startswith('2016.11'):
+        search_cx = '011515552685726825874:dlsj745pvhq' # 2016.11
+    else:
+        search_cx = '011515552685726825874:ht0p8miksrm' # latest
+else: # latest or something else
+    release = latest_release
     search_cx = '011515552685726825874:ht0p8miksrm' # latest
-elif release.startswith('2018.3'):
-    search_cx = '011515552685726825874:vadptdpvyyu' # 2018.3
-elif release.startswith('2017.7'):
-    search_cx = '011515552685726825874:w-hxmnbcpou' # 2017.7
-elif release.startswith('2016.11'):
-    search_cx = '011515552685726825874:dlsj745pvhq' # 2016.11
-else:
-    search_cx = '011515552685726825874:x17j5zl74g8'  # develop
 
 needs_sphinx = '1.3'
 
@@ -364,7 +377,7 @@ rst_prolog = """\
 
 # A shortcut for linking to tickets on the GitHub issue tracker
 extlinks = {
-    'blob': ('https://github.com/saltstack/salt/blob/%s/%%s' % 'develop', None),
+    'blob': ('https://github.com/saltstack/salt/blob/%s/%%s' % 'master', None),
     'issue': ('https://github.com/saltstack/salt/issues/%s', 'issue #'),
     'pull': ('https://github.com/saltstack/salt/pull/%s', 'PR #'),
     'formula_url': ('https://github.com/saltstack-formulas/%s', ''),
@@ -387,7 +400,7 @@ html_short_title = 'Salt'
 html_static_path = ['_static']
 html_logo = None # specified in the theme layout.html
 html_favicon = 'favicon.ico'
-html_use_smartypants = False
+smartquotes = False
 
 # Use Google customized search or use Sphinx built-in JavaScript search
 if on_saltstack:
