@@ -3,6 +3,7 @@
 # Import python libs
 from __future__ import absolute_import, print_function, unicode_literals
 import os
+import sys
 import uuid
 import hashlib
 import logging
@@ -21,6 +22,9 @@ from tests.support.helpers import (
 from tests.support.unit import skipIf
 import tests.support.paths as paths
 
+# Import 3rd party libs
+import salt.ext.six as six
+
 # Import salt libs
 import salt.utils.files
 import salt.utils.path
@@ -30,6 +34,9 @@ import salt.utils.stringutils
 log = logging.getLogger(__name__)
 
 
+SSL3_SUPPORT = sys.version_info >= (2, 7, 9)
+
+
 class CPModuleTest(ModuleCase):
     '''
     Validate the cp module
@@ -77,6 +84,7 @@ class CPModuleTest(ModuleCase):
         self.assertNotIn('bacon', data)
 
     @with_tempfile()
+    @skipIf(salt.utils.platform.is_windows() and six.PY3, 'This test hangs on Windows on Py3')
     def test_get_file_templated_paths(self, tgt):
         '''
         cp.get_file
@@ -277,6 +285,8 @@ class CPModuleTest(ModuleCase):
         self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
         self.assertNotIn('bacon', data)
 
+    @skipIf(not SSL3_SUPPORT, 'Requires python with SSL3 support')
+    @skipIf(salt.utils.platform.is_darwin() and six.PY2, 'This test hangs on OS X on Py2')
     @with_tempfile()
     def test_get_url_https(self, tgt):
         '''
@@ -295,6 +305,8 @@ class CPModuleTest(ModuleCase):
         self.assertIn('Windows', data)
         self.assertNotIn('AYBABTU', data)
 
+    @skipIf(not SSL3_SUPPORT, 'Requires python with SSL3 support')
+    @skipIf(salt.utils.platform.is_darwin() and six.PY2, 'This test hangs on OS X on Py2')
     def test_get_url_https_dest_empty(self):
         '''
         cp.get_url with https:// source given and destination omitted.
@@ -311,6 +323,8 @@ class CPModuleTest(ModuleCase):
         self.assertIn('Windows', data)
         self.assertNotIn('AYBABTU', data)
 
+    @skipIf(not SSL3_SUPPORT, 'Requires python with SSL3 support')
+    @skipIf(salt.utils.platform.is_darwin() and six.PY2, 'This test hangs on OS X on Py2')
     def test_get_url_https_no_dest(self):
         '''
         cp.get_url with https:// source given and destination set as None
@@ -385,6 +399,8 @@ class CPModuleTest(ModuleCase):
             ])
         self.assertEqual(ret, False)
 
+    @skipIf(not SSL3_SUPPORT, 'Requires python with SSL3 support')
+    @skipIf(salt.utils.platform.is_darwin() and six.PY2, 'This test hangs on OS X on Py2')
     def test_get_file_str_https(self):
         '''
         cp.get_file_str with https:// source given
@@ -665,3 +681,6 @@ class CPModuleTest(ModuleCase):
             self.assertTrue(os.path.isfile(tgt_cache_file), 'File was not cached on the master')
         finally:
             os.unlink(tgt_cache_file)
+
+    def test_envs(self):
+        self.assertEqual(sorted(self.run_function('cp.envs')), sorted(['base', 'prod']))
