@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Mike Place <mp@saltstack.com>`
+    :codeauthor: Mike Place <mp@saltstack.com>
 '''
 
 # Import python libs
@@ -16,11 +16,13 @@ from tests.support.helpers import skip_if_not_root
 # Import salt libs
 import salt.minion
 import salt.utils.event as event
-from salt.exceptions import SaltSystemExit
+from salt.exceptions import SaltSystemExit, SaltMasterUnresolvableError
 import salt.syspaths
 import tornado
+import tornado.testing
 from salt.ext.six.moves import range
 
+
 __opts__ = {}
 
 
@@ -102,33 +104,6 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                                                          'source_ip': '111.1.0.1',
                                                          'master_ip': '127.0.0.1'}
 
-    @skip_if_not_root
-    def test_sock_path_len(self):
-        '''
-        This tests whether or not a larger hash causes the sock path to exceed
-        the system's max sock path length. See the below link for more
-        information.
-
-        https://github.com/saltstack/salt/issues/12172#issuecomment-43903643
-        '''
-        opts = {
-            'id': 'salt-testing',
-            'hash_type': 'sha512',
-            'sock_dir': os.path.join(salt.syspaths.SOCK_DIR, 'minion'),
-            'extension_modules': ''
-        }
-        with patch.dict(__opts__, opts):
-            try:
-                event_publisher = event.AsyncEventPublisher(__opts__)
-                result = True
-            except ValueError:
-                #  There are rare cases where we operate a closed socket, especially in containers.
-                # In this case, don't fail the test because we'll catch it down the road.
-                result = True
-            except SaltSystemExit:
-                result = False
-        self.assertTrue(result)
-
     # Tests for _handle_decoded_payload in the salt.minion.Minion() class: 3
 
     def test_handle_decoded_payload_jid_match_in_jid_queue(self):
@@ -230,10 +205,10 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
             try:
 
                 # mock gen.sleep to throw a special Exception when called, so that we detect it
-                class SleepCalledEception(Exception):
+                class SleepCalledException(Exception):
                     """Thrown when sleep is called"""
                     pass
-                tornado.gen.sleep.return_value.set_exception(SleepCalledEception())
+                tornado.gen.sleep.return_value.set_exception(SleepCalledException())
 
                 # up until process_count_max: gen.sleep does not get called, processes are started normally
                 for i in range(process_count_max):
@@ -248,7 +223,7 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                 mock_data = {'fun': 'foo.bar',
                              'jid': process_count_max + 1}
 
-                self.assertRaises(SleepCalledEception,
+                self.assertRaises(SleepCalledException,
                                   lambda: io_loop.run_sync(lambda: minion._handle_decoded_payload(mock_data)))
                 self.assertEqual(salt.utils.process.SignalHandlingMultiprocessingProcess.start.call_count,
                                  process_count_max)
@@ -306,3 +281,89 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                 self.assertTrue('beacons' not in minion.periodic_callbacks)
             finally:
                 minion.destroy()
+
+    def test_when_ping_interval_is_set_the_callback_should_be_added_to_periodic_callbacks(self):
+        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
+                patch('salt.minion.Minion.sync_connect_master', MagicMock(side_effect=RuntimeError('stop execution'))), \
+                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
+            mock_opts = self.get_config('minion', from_scratch=True)
+            mock_opts['ping_interval'] = 10
+            io_loop = tornado.ioloop.IOLoop()
+            io_loop.make_current()
+            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+            try:
+                try:
+                    minion.connected = MagicMock(side_effect=(False, True))
+                    minion._fire_master_minion_start = MagicMock()
+                    minion.tune_in(start=False)
+                except RuntimeError:
+                    pass
+
+                # Make sure the scheduler is initialized but the beacons are not
+                self.assertTrue('ping' in minion.periodic_callbacks)
+            finally:
+                minion.destroy()
+
+    def test_minion_retry_dns_count(self):
+        '''
+        Tests that the resolve_dns will retry dns look ups for a maximum of
+        3 times before raising a SaltMasterUnresolvableError exception.
+        '''
+        with patch.dict(__opts__, {'ipv6': False, 'master': 'dummy',
+                                   'master_port': '4555',
+                                   'retry_dns': 1, 'retry_dns_count': 3}):
+            self.assertRaises(SaltMasterUnresolvableError,
+                              salt.minion.resolve_dns, __opts__)
+
+    def test_gen_modules_executors(self):
+        '''
+        Ensure gen_modules is called with the correct arguments #54429
+        '''
+        mock_opts = self.get_config('minion', from_scratch=True)
+        io_loop = tornado.ioloop.IOLoop()
+        io_loop.make_current()
+        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+
+        class MockPillarCompiler(object):
+            def compile_pillar(self):
+                return {}
+
+        try:
+            with patch('salt.pillar.get_pillar', return_value=MockPillarCompiler()):
+                with patch('salt.loader.executors') as execmock:
+                    minion.gen_modules()
+            assert execmock.called_with(minion.opts, minion.functions)
+        finally:
+            minion.destroy()
+
+
+@skipIf(NO_MOCK, NO_MOCK_REASON)
+class MinionAsyncTestCase(TestCase, AdaptedConfigurationTestCaseMixin, tornado.testing.AsyncTestCase):
+
+    @skip_if_not_root
+    def test_sock_path_len(self):
+        '''
+        This tests whether or not a larger hash causes the sock path to exceed
+        the system's max sock path length. See the below link for more
+        information.
+
+        https://github.com/saltstack/salt/issues/12172#issuecomment-43903643
+        '''
+        opts = {
+            'id': 'salt-testing',
+            'hash_type': 'sha512',
+            'sock_dir': os.path.join(salt.syspaths.SOCK_DIR, 'minion'),
+            'extension_modules': ''
+        }
+        with patch.dict(__opts__, opts):
+            try:
+                event_publisher = event.AsyncEventPublisher(__opts__)
+                result = True
+            except ValueError:
+                #  There are rare cases where we operate a closed socket, especially in containers.
+                # In this case, don't fail the test because we'll catch it down the road.
+                result = True
+            except SaltSystemExit:
+                result = False
+        self.assertTrue(result)
